@@ -1,21 +1,22 @@
 /* This program, double, is a filter for converting, scaling, cutting and
    joining data sets.  The data sets may be unformatted (i.e., in binary
-   double precision format), or in ascii format.  Copyright (C) 1989-1997
+   double precision format), or in ascii format.  Copyright (C) 1989-1998
    Free Software Foundation, Inc. */
 
 #include "sys-defines.h"
 #include "getopt.h"
 
+/* type of data in input and output streams */
 typedef enum
 {
-  T_DOUBLE, T_ASCII
+  T_ASCII, T_SINGLE, T_DOUBLE, T_INTEGER
 }
 data_type;
 
 data_type input_type = T_ASCII;
 data_type output_type = T_ASCII;
 
-char *progname = "double";
+const char *progname = "double"; /* name of this program */
 
 int precision = 8;		/* default no. of digits after decimal pt. */
 
@@ -48,54 +49,57 @@ struct option long_options[] =
   {NULL, 0, 0, 0}
 };
 
+/* null-terminated list of options that we don't show to the user */
+int hidden_options[] = { 0 };
+
 /* forward references */
-#if __STDC__
-#define P__(a)	a
-#else
-#define P__(a)	()
-#endif
-Boolean mung_dataset P__ ((FILE *input, int record_length, Boolean *field_array, double scale, double baseline, FILE *add_fp, FILE *mult_fp, FILE *pre_join_fp, FILE *post_join_fp, int precision, Boolean suppress));
-Boolean read_datum P__ ((FILE *input, double *dptr));
-Boolean skip_whitespace P__ ((FILE *stream));
-Boolean write_datum P__ ((double data, int precision));
-Voidptr xcalloc P__ ((unsigned int nmemb, unsigned int size));
-Voidptr xmalloc P__ ((unsigned int size));
-char *xstrdup P__ ((const char *s));
-int get_record P__ ((FILE *input, double *record, int record_length));
-void display_usage P__ ((void));
-void display_version P__ ((void));
-void open_file P__ ((char *name, FILE **fpp));
-void output_dataset_separator P__ ((void));
-void set_format_type P__ ((char *s, data_type *typep));
-#undef P__
+bool mung_dataset __P ((FILE *input, int record_length, bool *field_array, double scale, double baseline, FILE *add_fp, FILE *mult_fp, FILE *pre_join_fp, FILE *post_join_fp, int precision, bool suppress));
+bool read_float __P ((FILE *input, double *dptr));
+bool skip_whitespace __P ((FILE *stream));
+bool write_float __P ((double data, int precision));
+int get_record __P ((FILE *input, double *record, int record_length));
+void maybe_emit_oob_warning __P ((void));
+void open_file __P ((char *name, FILE **fpp));
+void output_dataset_separator __P ((void));
+void set_format_type __P ((char *s, data_type *typep));
+/* from libcommon */
+extern void display_usage __P((const char *progname, const int *omit_vals, bool files, bool fonts));
+extern void display_version __P((const char *progname)); 
+extern Voidptr xcalloc __P ((unsigned int nmemb, unsigned int size));
+extern Voidptr xmalloc __P ((unsigned int size));
+extern char *xstrdup __P ((const char *s));
 
 
 int
+#ifdef _HAVE_PROTOS
+main (int argc, char **argv)
+#else
 main (argc, argv)
      int argc;
      char **argv;
+#endif
 {
   int option;
   int opt_index;
   int errcnt = 0;		/* errors encountered */
-  Boolean show_version = FALSE;	/* remember to show version message */
-  Boolean show_usage = FALSE;	/* remember to output usage message */
+  bool show_version = false;	/* remember to show version message */
+  bool show_usage = false;	/* remember to output usage message */
   char *add_file = NULL, *mult_file = NULL;
   char *pre_join_file = NULL, *post_join_file = NULL;
   FILE *add_fp = NULL, *mult_fp = NULL; 
   FILE *pre_join_fp = NULL, *post_join_fp = NULL;
   double scale = 1.0, baseline = 0.0; /* mult., additive constants */
-  int record_length;
+  int record_length = 1;
   int record_min = 0, record_max = MAXINT, record_spacing = 1;
   int dataset_min = 0, dataset_max = MAXINT, dataset_spacing = 1;  
   int local_record_min, local_record_max, local_record_spacing;
   int local_dataset_min, local_dataset_max, local_dataset_spacing;  
   int field_array_len = 0;	/* initial size of field array */
   int dataset_index = 0;	/* running count */
-  Boolean *field_array = NULL, *new_field_array;
-  Boolean more_points, dataset_printed = FALSE;
+  bool *field_array = NULL, *new_field_array;
+  bool more_points, dataset_printed = false;
 
-  while (TRUE)
+  while (true)
     {
       option = getopt_long (argc, argv, "I:O:q:T:P:j:J:t:p:R:r::f::d::", long_options, &opt_index);
       if (option == 0)
@@ -106,10 +110,10 @@ main (argc, argv)
 	  /* ----------- options with no argument --------------*/
 
 	case 'V' << 8:		/* display version */
-	  show_version = TRUE;
+	  show_version = true;
 	  break;
 	case 'h' << 8:		/* help */
-	  show_usage = TRUE;
+	  show_usage = true;
 	  break;
 
 	  /* ----------- options with a single argument --------------*/
@@ -168,7 +172,7 @@ main (argc, argv)
 	      || (record_length < 1))
 	    {
 	      fprintf (stderr,
-		       "%s: bad record length `%s', must be integer >= 1\n",
+		       "%s: error: bad record length `%s', must be integer >= 1\n",
 		       progname, optarg);
 	      return 1;
 	    }
@@ -219,7 +223,7 @@ main (argc, argv)
 	  break;
 
 	case 'f':
-	  while (TRUE)
+	  while (true)
 	    {
 	      int field_index;
 
@@ -236,16 +240,16 @@ main (argc, argv)
 	      if (field_index >= field_array_len)
 		{
 		  new_field_array 
-		    = xcalloc ((unsigned)(field_index + 1), sizeof(Boolean));
+		    = (bool *)xcalloc ((unsigned)(field_index + 1), sizeof(bool));
 		  memcpy (new_field_array, field_array,
-			  field_array_len * sizeof(Boolean));
+			  field_array_len * sizeof(bool));
 		  if (field_array != NULL) /* NULL on entry */
 		    free (field_array);
 		  field_array = new_field_array;
 		  field_array_len = field_index + 1;
 		}
 
-	      field_array[field_index] = TRUE;
+	      field_array[field_index] = true;
 	      optind++;		/* tell getopt we recognized field index */
 	    }
 	  break;
@@ -266,17 +270,17 @@ main (argc, argv)
 				/* endwhile */
   if (errcnt > 0)
     {
-      fprintf (stderr, "Try `%s --help' for more information.\n", progname);
+      fprintf (stderr, "Try `%s --help' for more information\n", progname);
       return 1;
     }
   if (show_version)
     {
-      display_version ();
+      display_version (progname);
       return 0;
     }
   if (show_usage)
     {
-      display_usage ();
+      display_usage (progname, hidden_options, true, false);
       return 0;
     }
 
@@ -297,16 +301,16 @@ main (argc, argv)
   /* Sanity checks on user-supplied options */
 
   if (field_array_len == 0)
-    fprintf (stderr, "%s: warning: no fields specified\n", progname);
+    fprintf (stderr, "%s: no fields specified\n", progname);
   if (field_array_len > record_length)
     {
       fprintf (stderr, 
 	       "%s: error: at least one field index out of bounds\n", progname);
       return 1;
     }
-  new_field_array = xcalloc ((unsigned)record_length, sizeof(Boolean));
+  new_field_array = (bool *)xcalloc ((unsigned)record_length, sizeof(bool));
   memcpy (new_field_array, field_array,
-	  field_array_len * sizeof(Boolean));
+	  field_array_len * sizeof(bool));
   if (field_array != NULL) /* NULL originally */
     free (field_array);
   field_array = new_field_array;
@@ -330,12 +334,16 @@ main (argc, argv)
 	{
 	  FILE *data_fp;
 	  
-	  open_file (argv[optind], &data_fp);
+	  /* open file, treat "-" as stdin */
+	  if (strcmp (argv[optind], "-") == 0)
+	    data_fp = stdin;
+	  else
+	    open_file (argv[optind], &data_fp);
 
 	  /* loop through datasets in file (may be more than one) */
 	  do
 	    {
-	      Boolean dataset_ok;
+	      bool dataset_ok;
 	      
 	      dataset_ok = ((dataset_index >= dataset_min)
 			    && (dataset_index <= dataset_max)
@@ -354,20 +362,22 @@ main (argc, argv)
 					  precision, !dataset_ok);
 
 	      if (dataset_ok)
-		dataset_printed = TRUE;
+		dataset_printed = true;
 	      
 	      dataset_index++;
 	    } 
 	  while (more_points);
 	  
-	  fclose (data_fp);
+	  /* close file (but don't close stdin) */
+	  if (data_fp != stdin)
+	    fclose (data_fp);
 	}
     }
   else			/* no files spec'd, read stdin instead */
     /* loop through datasets (may be more than one) */
     do
       {
-	Boolean dataset_ok;
+	bool dataset_ok;
 	
 	dataset_ok = ((dataset_index >= dataset_min)
 		      && (dataset_index <= dataset_max)
@@ -385,7 +395,7 @@ main (argc, argv)
 				    pre_join_fp, post_join_fp,
 				    precision, !dataset_ok);
 	if (dataset_ok)
-	  dataset_printed = TRUE;
+	  dataset_printed = true;
 
 	dataset_index++;
       }
@@ -394,72 +404,23 @@ main (argc, argv)
   return 0;
 }
 
-void
-display_usage ()
-{
-  int i;
-  int col = 0;
-  
-  fprintf (stderr, "Usage: %s", progname);
-  col += (strlen (progname) + 7);
-  for (i = 0; long_options[i].name; i++)
-    {
-      int option_len;
-      
-      option_len = strlen (long_options[i].name);
-      if (col >= 80 - (option_len + 16))
-	{
-	  fprintf (stderr, "\n\t");
-	  col = 8;
-	}
-      fprintf (stderr, " [--%s", long_options[i].name);
-      col += (option_len + 4);
-      if ((unsigned int)(long_options[i].val) < 256)
-	{
-	  fprintf (stderr, " | -%c", long_options[i].val);
-	  col += 5;
-	}
-      if (long_options[i].has_arg == ARG_REQUIRED)
-	{
-	  fprintf (stderr, " arg]");
-	  col += 5;
-	}
-      else if (long_options[i].has_arg == ARG_OPTIONAL)
-	{
-	  fprintf (stderr, " [arg]]");
-	  col += 10;
-	}
-      else
-	{
-	  fprintf (stderr, "]");
-	  col++;
-	}
-    }
-  fprintf (stderr, "\n");
-}
-
-void
-display_version ()
-{
-  fprintf (stderr, "%s (GNU plotutils) %s\n", progname, VERSION);
-  fprintf (stderr, "Copyright (C) 1997 Free Software Foundation, Inc.\n");
-  fprintf (stderr, 
-	   "The GNU plotutils come with NO WARRANTY, to the extent permitted by law.\n");
-  fprintf (stderr, "You may redistribute copies of the GNU plotutils\n");
-  fprintf (stderr, "under the terms of the GNU General Public License.\n");
-}
-
-/* read_datum reads a single floating point quantity from an input stream
+/* read_float reads a single floating point quantity from an input stream
    (in either ascii or double format).  Return value indicates whether it
    was read successfully. */
 
-Boolean 
-read_datum (input, dptr)
+bool 
+#ifdef _HAVE_PROTOS
+read_float (FILE *input, double *dptr)
+#else
+read_float (input, dptr)
      FILE *input;
      double *dptr;
+#endif
 {
   int num_read;
   double dval;
+  float fval;
+  int ival;
 
   switch (input_type)
     {
@@ -467,16 +428,24 @@ read_datum (input, dptr)
     default:
       num_read = fscanf (input, "%lf", &dval);
       break;
+    case T_SINGLE:
+      num_read = fread ((Voidptr) &fval, sizeof (fval), 1, input);
+      dval = fval;
+      break;
     case T_DOUBLE:
       num_read = fread ((Voidptr) &dval, sizeof (dval), 1, input);
       break;
+    case T_INTEGER:
+      num_read = fread ((Voidptr) &ival, sizeof (ival), 1, input);
+      dval = ival;
+      break;
     }
   if (num_read <= 0)
-    return FALSE;
+    return false;
   else
     {
       *dptr = dval;
-      return TRUE;
+      return true;
     }
 }
 
@@ -487,20 +456,24 @@ read_datum (input, dptr)
    it indicates that an explicit end-of-dataset indicator was seen in the
    input stream.  For an ascii stream this is two newlines in succession;
    for a stream of doubles it is a MAXDOUBLE appearing at what would
-   otherwise have been the beginning of the record. */
+   otherwise have been the beginning of the record, etc. */
 
 int
+#ifdef _HAVE_PROTOS
+get_record (FILE *input, double *record, int record_length)
+#else
 get_record (input, record, record_length)
      FILE *input;
      double *record;
      int record_length;
+#endif
 {
-  Boolean success;
+  bool success;
   int i;
 
   if (input_type == T_ASCII)
     {
-      Boolean two_newlines;
+      bool two_newlines;
 
       /* skip whitespace, up to but not including 2nd newline */
       two_newlines = skip_whitespace (input);
@@ -515,14 +488,17 @@ get_record (input, record, record_length)
     {
       double val;
 
-      success = read_datum (input, &val);
-      if (i == 0 && input_type == T_DOUBLE && val == MAXDOUBLE)
+      success = read_float (input, &val);
+      if (i == 0 && 
+	  ((input_type == T_DOUBLE && val == MAXDOUBLE)
+	   || (input_type == T_SINGLE && val == (double)MAXFLOAT)
+	   || (input_type == T_INTEGER && val == (double)MAXINT)))
 	/* end-of-dataset indicator */
 	return 2;
       if (!success)		/* EOF or garbage */
 	{
 	  if (i > 0)
-	    fprintf (stderr, "%s: warning: premature EOF in input stream\n",
+	    fprintf (stderr, "%s: encountered premature EOF in input stream\n",
 		     progname);
 	  return 1;
 	}
@@ -532,33 +508,66 @@ get_record (input, record, record_length)
   return 0;
 }
 
-Boolean
-write_datum (datum, precision)
-     double datum;
+/* Emit a double, in specified output representation.  Be sure to inform
+   user if any of the emitted values were out-of-bounds for
+   single-precision or integer format. */
+bool
+#ifdef _HAVE_PROTOS
+write_float (double x, int precision)
+#else
+write_float (x, precision)
+     double x;
      int precision;
+#endif
 {
   int num_written = 0;
+  float fx;
+  int ix;
 
   switch (output_type)
     {
-    case T_DOUBLE:
-      num_written = fwrite ((Voidptr) &datum, sizeof (datum), 1, stdout);
-      break;
     case T_ASCII:
     default:
-      num_written = printf ("%.*g\n", precision, datum);
+      num_written = printf ("%.*g\n", precision, x);
+      break;
+    case T_SINGLE:
+      fx = FROUND(x);
+      if (fx == MAXFLOAT || fx == -(MAXFLOAT))
+	{
+	  maybe_emit_oob_warning();
+	  if (fx == MAXFLOAT)
+	    fx *= 0.99999;	/* kludge */
+	}
+      num_written = fwrite ((Voidptr) &fx, sizeof (fx), 1, stdout);
+      break;
+    case T_DOUBLE:
+      num_written = fwrite ((Voidptr) &x, sizeof (x), 1, stdout);
+      break;
+    case T_INTEGER:
+      ix = IROUND(x);
+      if (ix == MAXINT || ix == -(MAXINT))
+	{
+	  maybe_emit_oob_warning();
+	  if (ix == MAXINT)
+	    ix--;
+	}
+      num_written = fwrite ((Voidptr) &ix, sizeof (ix), 1, stdout);
       break;
     }
   if (num_written < 0)
-    return FALSE;
+    return false;
   else
-    return TRUE;
+    return true;
 }
 
 void
+#ifdef _HAVE_PROTOS
+open_file (char *name, FILE **fpp)
+#else
 open_file (name, fpp)
      char *name;
      FILE **fpp;
+#endif
 {
   FILE *fp;
 
@@ -572,9 +581,13 @@ open_file (name, fpp)
 }
 
 void
+#ifdef _HAVE_PROTOS
+set_format_type (char *s, data_type *typep)
+#else
 set_format_type (s, typep)
      char *s;
      data_type *typep;
+#endif
 {
   switch (s[0])
     {
@@ -582,10 +595,18 @@ set_format_type (s, typep)
     case 'A':
       *typep = T_ASCII;
       break;
+    case 'f':
+    case 'F':
+      *typep = T_SINGLE;
+      break;
     case 'd':
     case 'D':
       *typep = T_DOUBLE;
       break;
+    case 'i':
+    case 'I':
+      *typep = T_INTEGER;
+      break;
     default:
       {
 	fprintf (stderr, "%s: error: invalid data format type `%s'\n",
@@ -596,67 +617,31 @@ set_format_type (s, typep)
     }
 }
 
-Voidptr 
-xmalloc (size)
-     unsigned int size;
-{
-  Voidptr p;
-  p = (Voidptr) malloc (size);
-
-  if (p == (Voidptr) NULL)
-    {
-      fprintf (stderr, "%s: ", progname);
-      perror ("malloc failed");
-      exit (1);
-    }
-  return p;
-}
-
-Voidptr 
-xcalloc (nmemb, size)
-     unsigned int nmemb, size;
-{
-  Voidptr p;
-  p = (Voidptr) calloc (nmemb, size);
-
-  if (p == (Voidptr) NULL)
-    {
-      fprintf (stderr, "%s: ", progname);
-      perror ("calloc failed");
-      exit (1);
-    }
-  return p;
-}
-
-char *
-xstrdup (s)
-     const char *s;
-{
-  char *t = (char *)xmalloc (strlen (s) + 1);
-
-  strcpy (t, s);
-  return t;
-}
-
 /* mung_dataset() is the main routine for extracting fields from records in
    a dataset, and munging them.  Its return value indicates whether the
    records in the input stream ended with an explicit end-of-dataset
    indicator, i.e., whether another dataset is expected to follow.  An
    end-of-dataset indicator is two newlines in succession for an ascii
-   stream, and a MAXDOUBLE for a stream of doubles. */
-
-Boolean
+   stream, and a MAXDOUBLE for a stream of doubles, etc. */
+bool
+#ifdef _HAVE_PROTOS
+mung_dataset (FILE *input, int record_length, bool *field_array, 
+	      double scale, double baseline, FILE *add_fp, FILE *mult_fp, 
+	      FILE *pre_join_fp, FILE *post_join_fp, int precision, 
+	      bool suppress)
+#else
 mung_dataset (input, record_length, field_array, scale, baseline, add_fp, mult_fp, pre_join_fp, post_join_fp, precision, suppress)
      FILE *input;
      int record_length;		/* number of fields per record in dataset */
-     Boolean *field_array;	/* fields we'll extract */
+     bool *field_array;	/* fields we'll extract */
      double scale, baseline;
      FILE *add_fp, *mult_fp, *pre_join_fp, *post_join_fp;
      int precision;
-     Boolean suppress;		/* suppress output for this dataset */
+     bool suppress;		/* suppress output for this dataset */
+#endif
 {
   double *record = (double *)xmalloc (record_length * sizeof(double));
-  Boolean in_trouble = suppress; /* once in trouble, we never get out */
+  bool in_trouble = suppress; /* once in trouble, we never get out */
   
   if (!in_trouble)
     {
@@ -671,22 +656,22 @@ mung_dataset (input, record_length, field_array, scale, baseline, add_fp, mult_f
 	fseek(post_join_fp, 0L, 0);
     }
 
-  while (TRUE)
+  while (true)
     {
       int i;
       int success;
       double add_data, mult_data, pre_join_data, post_join_data;
 
-      if (!in_trouble && add_fp && read_datum (add_fp, &add_data) == FALSE)
-	in_trouble = TRUE;
-      if (!in_trouble && mult_fp && read_datum (mult_fp, &mult_data) == FALSE)
-	in_trouble = TRUE;
+      if (!in_trouble && add_fp && read_float (add_fp, &add_data) == false)
+	in_trouble = true;
+      if (!in_trouble && mult_fp && read_float (mult_fp, &mult_data) == false)
+	in_trouble = true;
       if (!in_trouble && pre_join_fp 
-	  && read_datum (pre_join_fp, &pre_join_data) == FALSE)
-	in_trouble = TRUE;
+	  && read_float (pre_join_fp, &pre_join_data) == false)
+	in_trouble = true;
       if (!in_trouble && post_join_fp 
-	  && read_datum (post_join_fp, &post_join_data) == FALSE)
-	in_trouble = TRUE;
+	  && read_float (post_join_fp, &post_join_data) == false)
+	in_trouble = true;
   
       success = get_record (input, record, record_length);
 
@@ -697,10 +682,10 @@ mung_dataset (input, record_length, field_array, scale, baseline, add_fp, mult_f
 	    continue;
 
 	  if (pre_join_fp)
-	    write_datum (pre_join_data, precision);
+	    write_float (pre_join_data, precision);
 
 	  for (i = 0; i < record_length; i++)
-	    if (field_array[i] == TRUE)
+	    if (field_array[i] == true)
 	      {
 		double datum;
 		
@@ -713,17 +698,17 @@ mung_dataset (input, record_length, field_array, scale, baseline, add_fp, mult_f
 		datum += baseline;
 
 		/* output the munged datum */
-		write_datum (datum, precision);
+		write_float (datum, precision);
 	      }
 	      
 	  if (post_join_fp)
-	    write_datum (post_join_data, precision);
+	    write_float (post_join_data, precision);
 
 	  break;
 	case 1:			/* no more records, EOF seen */
-	  return FALSE;
+	  return false;
 	case 2:			/* end of dataset, but input continues */
-	  return TRUE;
+	  return true;
 	}
     }      
 }	  
@@ -733,9 +718,13 @@ mung_dataset (input, record_length, field_array, scale, baseline, add_fp, mult_f
    whether or not two newlines were in fact seen.  (For ascii-format
    input streams, two newlines signals an end-of-dataset.) */
 
-Boolean
+bool
+#ifdef _HAVE_PROTOS
+skip_whitespace (FILE *stream)
+#else
 skip_whitespace (stream)
      FILE *stream;
+#endif
 {
   int lookahead;
   int nlcount = 0;
@@ -751,7 +740,7 @@ skip_whitespace (stream)
 	 && nlcount < 2);
 
   if (lookahead == EOF)
-    return FALSE;
+    return false;
   
   ungetc (lookahead, stream);
   return (nlcount == 2);
@@ -760,14 +749,19 @@ skip_whitespace (stream)
 /* Output a separator between datasets.  For ascii-format output streams
    this is an extra newline (after the one that the spline ended with,
    yielding two newlines in succession).  For double-format output streams
-   this is a MAXDOUBLE. */
+   this is a MAXDOUBLE, etc. */
 
 void
+#ifdef _HAVE_PROTOS
+output_dataset_separator(void)
+#else
 output_dataset_separator()
+#endif
 {
-  double dummy;
-  
-  dummy = MAXDOUBLE;
+  double ddummy;
+  float fdummy;
+  int idummy;
+
   switch (output_type)
     {
     case T_ASCII:
@@ -775,8 +769,32 @@ output_dataset_separator()
       printf ("\n");
       break;
     case T_DOUBLE:
-      fwrite ((Voidptr) &dummy, 
-	      sizeof(dummy), 1, stdout);
+      ddummy = MAXDOUBLE;
+      fwrite ((Voidptr) &ddummy, sizeof(ddummy), 1, stdout);
+      break;
+    case T_SINGLE:
+      fdummy = MAXFLOAT;
+      fwrite ((Voidptr) &fdummy, sizeof(fdummy), 1, stdout);
+      break;
+    case T_INTEGER:
+      idummy = MAXINT;
+      fwrite ((Voidptr) &idummy, sizeof(idummy), 1, stdout);
       break;
     }
 }
+
+void
+#ifdef HAVE_PROTOS
+maybe_emit_oob_warning (void)
+#else
+maybe_emit_oob_warning ()
+#endif
+{
+  static bool warning_written = false;
+
+  if (!warning_written)
+    {
+      fprintf (stderr, "%s: approximating one or more out-of-bounds output values\n", progname);
+      warning_written = true;
+    }
+}
